@@ -19,7 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/signal.h>
 
 #include <stddef.h>
@@ -35,7 +37,10 @@
 #include "config.h"
 #include "seahorse-agent.h"
 #include "seahorse-gpg-options.h"
+#include "seahorse-passphrase.h"
+#include "seahorse-pgp-key.h"
 
+gboolean seahorse_agent_enabled = FALSE;
 gboolean seahorse_agent_displayvars = FALSE;
 gboolean seahorse_agent_cshell = FALSE;
 
@@ -121,6 +126,16 @@ unprocess_gpg_conf ()
 void
 seahorse_agent_prefork ()
 {
+    /* Detect and see if there's an agent */
+    switch (seahorse_passphrase_detect_agent (SKEY_PGP)) {
+    case SEAHORSE_AGENT_NONE:
+        break;
+    default:
+        g_message ("Another GPG agent already running\n");
+        return;
+    }
+    
+    seahorse_agent_enabled = TRUE;
     if (seahorse_agent_io_socket () == -1)
         _exit (1); /* Message already printed */
 }
@@ -128,7 +143,12 @@ seahorse_agent_prefork ()
 void
 seahorse_agent_postfork (pid_t child)
 {
-    const gchar *socket = seahorse_agent_io_get_socket ();
+    const gchar *socket;
+
+    if(!seahorse_agent_enabled)
+        return;
+    
+    socket = seahorse_agent_io_get_socket ();
     g_return_if_fail (socket != NULL);
     
     /* If any of these fail, they simply exit */
@@ -138,22 +158,28 @@ seahorse_agent_postfork (pid_t child)
         process_gpg_conf (socket, child);    
 }
 
-int
+gboolean
 seahorse_agent_init ()
 {
+    if(!seahorse_agent_enabled)
+        return TRUE;
+    
     if (seahorse_agent_io_init () == -1)
-        return -1;               /* message already printed */
+        return FALSE;           /* message already printed */
     
     /* Initialize our sub systems */
     seahorse_agent_actions_init ();
     seahorse_agent_cache_init ();
     
-    return 0;
+    return TRUE;
 }
 
 void
 seahorse_agent_uninit ()
 {
+    if(!seahorse_agent_enabled)
+        return;
+
     if (!seahorse_agent_displayvars)
         unprocess_gpg_conf ();
         
@@ -166,3 +192,4 @@ seahorse_agent_uninit ()
     seahorse_agent_actions_uninit ();
     seahorse_agent_io_uninit ();
 }
+
