@@ -34,10 +34,30 @@
  * seahorse-agent gconf keys
  */
 
-#define SETTING_AUTH 		"/apps/seahorse/agent/cache_authorize"
-#define SETTING_CACHE       "/apps/seahorse/agent/cache_enabled"
-#define SETTING_EXPIRE      "/apps/seahorse/agent/cache_expire"
-#define SETTING_TTL         "/apps/seahorse/agent/cache_ttl"
+#define AGENT_SETTINGS      "/apps/seahorse/agent"
+#define SETTING_CACHE       AGENT_SETTINGS "/cache_enabled"
+#define SETTING_METHOD      AGENT_SETTINGS "/cache_method"
+#define SETTING_TTL         AGENT_SETTINGS "/cache_ttl"
+#define SETTING_EXPIRE      AGENT_SETTINGS "/cache_expire"
+#define SETTING_AUTH        AGENT_SETTINGS "/cache_authorize"
+#define SETTING_AGENT_SSH   AGENT_SETTINGS "/load_ssh"
+#define METHOD_GNOME        "gnome"
+
+/* -----------------------------------------------------------------------------
+ * seahorse-agent.c
+ */
+
+/* Called from the original process before and after fork */
+void seahorse_agent_prefork ();
+void seahorse_agent_postfork (pid_t child);
+
+/* Called in the new child process */
+gboolean seahorse_agent_init ();
+void seahorse_agent_uninit ();
+
+/* Global options to set from the command line */
+extern gboolean seahorse_agent_displayvars;
+extern gboolean seahorse_agent_cshell;
 
 /* -----------------------------------------------------------------------------
  * seahorse-agent-io.c
@@ -46,7 +66,8 @@
 struct _SeahorseAgentConn;
 typedef struct _SeahorseAgentConn SeahorseAgentConn;
 
-int seahorse_agent_io_socket (const char **socketname);
+int seahorse_agent_io_socket ();
+const gchar* seahorse_agent_io_get_socket ();
 int seahorse_agent_io_init ();
 void seahorse_agent_io_uninit ();
 void seahorse_agent_io_reply (SeahorseAgentConn *rq, gboolean ok, const gchar *response);
@@ -55,7 +76,6 @@ void seahorse_agent_io_reply (SeahorseAgentConn *rq, gboolean ok, const gchar *r
  * seahorse-agent-actions.c
  */
 
-
 typedef struct _SeahorseAgentPassReq {
     const gchar *id;
     const gchar *errmsg;
@@ -80,14 +100,12 @@ void seahorse_agent_actions_nextgui ();
 void seahorse_agent_cache_init ();
 void seahorse_agent_cache_uninit ();
 const gchar *seahorse_agent_cache_get (const gchar *id);
-void seahorse_agent_cache_set (const gchar *id, const gchar *pass,
-                               gboolean encode, gboolean lock);
+void seahorse_agent_cache_set (const gchar *id, const gchar *pass, gboolean lock);
 gboolean seahorse_agent_cache_has (const gchar *id, gboolean lock);
 void seahorse_agent_cache_clear (const gchar *id);
 void seahorse_agent_cache_clearall ();
 guint seahorse_agent_cache_count ();
-void seahorse_agent_cache_enum (GHFunc func, gpointer user_data);
-gchar *seahorse_agent_cache_getname (const gchar *id);
+GList* seahorse_agent_cache_get_keys ();
 
 /* -----------------------------------------------------------------------------
  * seahorse-agent-prompt.c
@@ -105,4 +123,16 @@ void seahorse_agent_prompt_cleanup ();
 void seahorse_agent_status_cleanup ();
 void seahorse_agent_status_update ();
 
-#endif                          /* __SEAHORSE_AGENT_H__ */
+/* -----------------------------------------------------------------------------
+ * seahorse-agent-ssh.c
+ */
+ 
+#define SEAHORSE_SSH_PING_MSG             252
+
+gboolean seahorse_agent_ssh_init ();
+GList*   seahorse_agent_ssh_cached_keys ();
+guint    seahorse_agent_ssh_count_keys ();
+void     seahorse_agent_ssh_clearall ();
+void     seahorse_agent_ssh_uninit ();
+
+#endif /* __SEAHORSE_AGENT_H__ */
