@@ -30,6 +30,8 @@
 #include "seahorse-hkp-source.h"
 #include "seahorse-util.h"
 #include "seahorse-pgp-key.h"
+#include "seahorse-gconf.h"
+#include "seahorse-vfs-data.h"
 
 #ifdef WITH_HKP
 
@@ -69,6 +71,13 @@
     
 #define HKP_ERROR_DOMAIN (get_hkp_error_domain())
 
+#define GCONF_USE_HTTP_PROXY    "/system/http_proxy/use_http_proxy"
+#define GCONF_HTTP_PROXY_HOST   "/system/http_proxy/host"
+#define GCONF_PROXY_PORT        "/system/http_proxy/port"
+#define GCONF_USE_AUTH          "/system/http_proxy/use_authentication"
+#define GCONF_AUTH_USER         "/system/http_proxy/authentication_user"
+#define GCONF_AUTH_PASS         "/system/http_proxy/authentication_password"
+
 static GQuark
 get_hkp_error_domain ()
 {
@@ -119,7 +128,7 @@ dump_soup_request (SoupMessage *msg)
     
     if (msg->request.body) {
         t = g_strndup (msg->request.body, msg->request.length);
-        g_printerr ("request: %s\n", t);
+        g_printerr ("request: %s\n", t ? t : "");
         g_free (t);
     }    
 }
@@ -135,7 +144,7 @@ dump_soup_response (SoupMessage *msg)
     
     if (msg->response.body) {
         t = g_strndup (msg->response.body, msg->response.length);
-        g_printerr ("response: %s\n", t);
+        g_printerr ("response: %s\n", t ? t : "");
         g_free (t);
     }    
 }
@@ -160,6 +169,7 @@ DECLARE_OPERATION (HKP, hkp)
     SoupSession *session;           /* The HTTP session */
     guint total;                    /* Number of requests queued */
     guint requests;                 /* Number of requests remaining */
+    gboolean cancelling;            /* Cancelling the request, don't process */
 END_DECLARE_OPERATION
 
 IMPLEMENT_OPERATION (HKP, hkp)
@@ -167,7 +177,42 @@ IMPLEMENT_OPERATION (HKP, hkp)
 static void 
 seahorse_hkp_operation_init (SeahorseHKPOperation *hop)
 {
-    hop->session = soup_session_async_new ();
+    SoupUri *uri;
+    gchar *host;
+    gchar *suri;
+    
+    if (seahorse_gconf_get_boolean (GCONF_USE_HTTP_PROXY)) {
+        
+        host = seahorse_gconf_get_string (GCONF_HTTP_PROXY_HOST);
+        if (host) {
+            
+            suri = g_strdup_printf ("http://%s/", host);
+            g_free (host);
+            
+            uri = soup_uri_new (suri);
+            
+            if (!uri) {
+                g_warning ("creation of SoupUri from '%s' failed", suri);
+                
+            } else {
+                
+                uri->port = seahorse_gconf_get_integer (GCONF_PROXY_PORT);
+                
+                if (seahorse_gconf_get_boolean (GCONF_USE_AUTH)) {
+                    uri->user = seahorse_gconf_get_string (GCONF_AUTH_USER);
+                    uri->passwd = seahorse_gconf_get_string (GCONF_AUTH_PASS);
+                }
+                
+                hop->session = soup_session_async_new_with_options (SOUP_SESSION_PROXY_URI, uri, NULL);
+            }
+
+            g_free (suri);
+        }
+    }
+    
+    /* Without a proxy */
+    if (!hop->session)
+        hop->session = soup_session_async_new ();
 }
 
 static void 
@@ -207,9 +252,12 @@ seahorse_hkp_operation_cancel (SeahorseOperation *operation)
     g_assert (SEAHORSE_IS_HKP_OPERATION (operation));
     hop = SEAHORSE_HKP_OPERATION (operation);
     
+    g_return_if_fail (seahorse_operation_is_running (operation));
+    hop->cancelling = TRUE;
+    
     if (hop->session != NULL) 
         soup_session_abort (hop->session);
-
+    
     seahorse_operation_mark_done (operation, TRUE, NULL);
 }
 
@@ -224,6 +272,8 @@ dehtmlize(gchar *line)
      */
     int parsedindex = 0;
     gchar *parsed = line;
+    
+    g_return_if_fail (line);
 
     while (*line != '\0') {
         switch(*line) {
@@ -286,6 +336,9 @@ fail_hkp_operation (SeahorseHKPOperation *hop, SoupMessage *msg, const gchar *te
 {
     gchar *t, *server;
     GError *error = NULL;
+    
+    if (!seahorse_operation_is_running (SEAHORSE_OPERATION (hop)))
+        return;
 
     g_object_get (hop->hsrc, "key-server", &server, NULL);
 
@@ -295,12 +348,14 @@ fail_hkp_operation (SeahorseHKPOperation *hop, SoupMessage *msg, const gchar *te
     } else if (msg) {
         /* Make the body lower case, and no tags */
         t = g_strndup (msg->response.body, msg->response.length);
-        dehtmlize (t);        
-        seahorse_util_string_lower (t);
+        if (t != NULL) {
+            dehtmlize (t);        
+            seahorse_util_string_lower (t);
+        }
 
-        if (strstr (t, "no keys"))
+        if (t && strstr (t, "no keys"))
             error = g_error_new (HKP_ERROR_DOMAIN, 0, _("No matching keys found on server '%s'."), server);
-        else if (strstr (t, "too many"))
+        else if (t && strstr (t, "too many"))
             error = g_error_new (HKP_ERROR_DOMAIN, 0, _("Search was not specific enough. Server '%s' found too many keys."), server);
         else 
             error = g_error_new (HKP_ERROR_DOMAIN, msg->status_code, _("Couldn't communicate with server '%s': %s"),
@@ -480,6 +535,9 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     GList *keys, *k;
     gchar *t;
     
+    if (hop->cancelling)
+        return;
+    
     DEBUG_HKP (("[hkp] Search Result:\n"));
     DEBUG_RESPONSE (msg);
     
@@ -503,8 +561,8 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Searching for keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Searching for keys..."), 
+                                               hop->requests, hop->total);
 }
 
 static gchar*
@@ -516,7 +574,10 @@ get_send_result (const gchar *response, unsigned int length)
     gboolean is_error = FALSE;
     
     text = g_strndup (response, length);
-    lines = g_strsplit (text, "\n", 0);    
+    if (!text)
+        return g_strdup ("");
+    
+    lines = g_strsplit (text, "\n", 0); 
 
     for (l = lines; *l; l++) {
 
@@ -552,6 +613,9 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 {
     gchar *errmsg;
 
+    if (hop->cancelling)
+        return;
+
     DEBUG_HKP (("[hkp] Send Result:\n"));
     DEBUG_RESPONSE (msg);
     
@@ -573,8 +637,8 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Uploading keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Uploading keys..."), 
+                                               hop->requests, hop->total);
 }
 
 gboolean
@@ -603,12 +667,16 @@ detect_key (const gchar *text, gint len, const gchar **start, const gchar **end)
 static void 
 get_callback (SoupMessage *msg, SeahorseHKPOperation *hop) 
 {
+    GError *err = NULL;
     const gchar *start;
     const gchar *end;
     gpgme_data_t data;
     const gchar *text;
+    gboolean ret;
     guint len;
-    int r;
+    
+    if (hop->cancelling)
+        return;
     
     DEBUG_HKP (("[hkp] Get Result:\n"));
     DEBUG_RESPONSE (msg);
@@ -630,82 +698,40 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
             break;
         
         /* Any key blocks get written to our result data */
-        data = (gpgme_data_t)g_object_get_data (G_OBJECT (hop), "result");
+        data = (gpgme_data_t)seahorse_operation_get_result (SEAHORSE_OPERATION (hop));
         g_return_if_fail (data != NULL);
             
-        r = gpgme_data_write (data, start, end - start);
-        g_return_if_fail (r != -1);
-
-        r = gpgme_data_write (data, "\n", 1);
-        g_return_if_fail (r != -1);
+        ret = seahorse_vfs_data_write_all (data, start, end - start, &err) &&
+              seahorse_vfs_data_write_all (data, "\n", -1, &err);
+        
+        if (!ret) {
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, err);
+            return;
+        }
     }
         
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Retrieving keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Retrieving keys..."), 
+                                               hop->requests, hop->total);
 }
 
 /* -----------------------------------------------------------------------------
  *  SEAHORSE HKP SOURCE
  */
- 
-/* GObject handlers */
-static void seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass);
 
-/* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_hkp_source_load       (SeahorseKeySource *src,
-                                                           SeahorseKeySourceLoad load,
-                                                           const gchar *key);
-static SeahorseOperation*  seahorse_hkp_source_import     (SeahorseKeySource *sksrc, 
-                                                           gpgme_data_t data);
-static SeahorseOperation*  seahorse_hkp_source_export     (SeahorseKeySource *sksrc, 
-                                                           GList *keys,     
-                                                           gboolean complete,
-                                                           gpgme_data_t data);
+G_DEFINE_TYPE (SeahorseHKPSource, seahorse_hkp_source, SEAHORSE_TYPE_SERVER_SOURCE);
 
-static SeahorseKeySourceClass *parent_class = NULL;
-
-GType
-seahorse_hkp_source_get_type (void)
+static void 
+seahorse_hkp_source_init (SeahorseHKPSource *hsrc)
 {
-    static GType type = 0;
+
+}
  
-    if (!type) {
-        
-        static const GTypeInfo tinfo = {
-            sizeof (SeahorseHKPSourceClass), NULL, NULL,
-            (GClassInitFunc) seahorse_hkp_source_class_init, NULL, NULL,
-            sizeof (SeahorseHKPSource), 0, NULL
-        };
-        
-        type = g_type_register_static (SEAHORSE_TYPE_SERVER_SOURCE, 
-                                       "SeahorseHKPSource", &tinfo, 0);
-    }
-  
-    return type;
-}
-
-/* Initialize the basic class stuff */
-static void
-seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
-{
-    SeahorseKeySourceClass *key_class;
-   
-    key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
-    key_class->load = seahorse_hkp_source_load;
-    key_class->import = seahorse_hkp_source_import;
-    key_class->export = seahorse_hkp_source_export;
-
-    parent_class = g_type_class_peek_parent (klass);
-}
-
 static SeahorseOperation*
-seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
-                          const gchar *match)
+seahorse_hkp_source_search (SeahorseKeySource *src, const gchar *match)
 {
-    SeahorseOperation *op;
     SeahorseHKPOperation *hop;
     SoupMessage *message;
     gchar *pattern = NULL;
@@ -713,23 +739,8 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     
     g_assert (SEAHORSE_IS_KEY_SOURCE (src));
     g_assert (SEAHORSE_IS_HKP_SOURCE (src));
-    
-    op = parent_class->load (src, load, match);
-    if (op != NULL)
-        return op;
-
-    /* No way to find new all or new keys */
-    if (load == SKSRC_LOAD_NEW || load == SKSRC_LOAD_ALL)
-        return seahorse_operation_new_complete (NULL);
-        
-    else if(load == SKSRC_LOAD_SEARCH)
-        pattern = soup_uri_encode (match, "+=/\\()");
-        
-    /* Load a specific key */
-    else if(load == SKSRC_LOAD_KEY) 
-        /* TODO: Does this actually work? */
-        pattern = soup_uri_encode (match, NULL);    
 
+    pattern = soup_uri_encode (match, "+=/\\()");
     g_return_val_if_fail (pattern != NULL, NULL);
     
     hop = setup_hkp_operation (SEAHORSE_HKP_SOURCE (src));
@@ -752,7 +763,7 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     
     hop->total = hop->requests = 1;
     t = g_strdup_printf (_("Searching for keys on: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);    
@@ -763,6 +774,26 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     return SEAHORSE_OPERATION (hop);
 }
 
+static SeahorseOperation*
+seahorse_hkp_source_load (SeahorseKeySource *src, GQuark keyid)
+{
+    SeahorseOperation *op;
+    
+    g_assert (SEAHORSE_IS_KEY_SOURCE (src));
+    g_assert (SEAHORSE_IS_HKP_SOURCE (src));
+    
+    op = SEAHORSE_KEY_SOURCE_CLASS (seahorse_hkp_source_parent_class)->load (src, keyid);
+    if (op != NULL)
+        return op;
+
+    /* No way to find new all or new keys */
+    if (!keyid)
+        return seahorse_operation_new_complete (NULL);
+
+    /* TODO: Does this actually work? */
+    return seahorse_hkp_source_search (src, seahorse_key_get_rawid (keyid));        
+}
+
 static SeahorseOperation* 
 seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
 {
@@ -825,32 +856,34 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
 
     hop->total = hop->requests;
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);
     g_free (uri);
     
     seahorse_util_string_slist_free (keydata);
+    gpgmex_data_release (data);
     
     return SEAHORSE_OPERATION (hop);
 }
 
-static SeahorseOperation* 
-seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                            gboolean complete, gpgme_data_t data)
+static SeahorseOperation*  
+seahorse_hkp_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids,
+                                gpgme_data_t data)
 {
     SeahorseHKPOperation *hop;
     SeahorseHKPSource *hsrc;
     SoupMessage *message;
     gchar *t, *server, *uri;
     const gchar *fpr;
-    guint l;
+    guint len;
+    GSList *l;
     
     g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (sksrc), NULL);
     hsrc = SEAHORSE_HKP_SOURCE (sksrc);
     
-    if (g_list_length (keys) == 0)
+    if (g_slist_length (keyids) == 0)
         return seahorse_operation_new_complete (NULL);
 
     server = get_http_server_address (sksrc);
@@ -861,23 +894,21 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
     if (data) {
         /* Note that we don't auto-free this */
-        g_object_set_data (G_OBJECT (hop), "result", data);
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, NULL);
     } else {
         /* But when we auto create a data object then we free it */
-        gpgme_data_new (&data);
-        g_return_val_if_fail (data != NULL, NULL);
-        g_object_set_data_full (G_OBJECT (hop), "result", data, 
-                                (GDestroyNotify)gpgme_data_release);
+        data = gpgmex_data_new ();
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, 
+                                        (GDestroyNotify)gpgmex_data_release);
     }
     
-    for ( ; keys; keys = g_list_next (keys)) {
-        g_assert (SEAHORSE_IS_KEY (keys->data));
+    for (l = keyids; l; l = g_slist_next (l)) {
 
         /* Get the key id and limit it to 8 characters */
-        fpr = seahorse_key_get_keyid (SEAHORSE_KEY (keys->data));
-        l = strlen (fpr);
-        if (l > 8)
-            fpr += (l - 8);
+        fpr = (const char*)seahorse_key_get_rawid (GPOINTER_TO_UINT (l->data));
+        len = strlen (fpr);
+        if (len > 8)
+            fpr += (len - 8);
 
         /* The get key URI */
         uri = g_strdup_printf ("http://%s/pks/lookup?op=get&search=0x%s", 
@@ -897,13 +928,29 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
     
     hop->total = hop->requests;
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);
     return SEAHORSE_OPERATION (hop);    
 }
 
+/* Initialize the basic class stuff */
+static void
+seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
+{
+    SeahorseKeySourceClass *key_class;
+   
+    key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
+    key_class->load = seahorse_hkp_source_load;
+    key_class->search = seahorse_hkp_source_search;
+    key_class->import = seahorse_hkp_source_import;
+    key_class->export_raw = seahorse_hkp_source_export_raw;
+
+    seahorse_hkp_source_parent_class = g_type_class_peek_parent (klass);
+}
+
+
 /**
  * seahorse_hkp_source_new
  * @uri: The server to connect to 
