@@ -23,6 +23,7 @@
 #include <gnome.h>
 
 #include "seahorse-key.h"
+#include "seahorse-context.h"
 #include "seahorse-key-source.h"
 
 gboolean 
@@ -53,6 +54,8 @@ enum {
     PROP_0,
     PROP_KEY_SOURCE,
     PROP_KEY_ID,
+    PROP_RAW_ID,
+    PROP_KEY_DESC,
     PROP_KTYPE,
     PROP_ETYPE,
     PROP_FLAGS,
@@ -62,6 +65,7 @@ enum {
 
 /* Special fields */
 enum {
+    PROP_SPECIAL_KEY_ID,
     PROP_SPECIAL_DISPLAY_NAME,
     PROP_SPECIAL_CN,
     PROP_SPECIAL_MAX
@@ -117,6 +121,12 @@ seahorse_key_get_property (GObject *object, guint prop_id, GValue *value,
     case PROP_KEY_ID:
         g_value_set_uint (value, skey->keyid);
         break;
+    case PROP_RAW_ID:
+        g_value_set_string (value, skey->rawid);
+        break;
+    case PROP_KEY_DESC:
+        g_value_set_string (value, skey->keydesc);
+        break;
     case PROP_KTYPE:
         g_value_set_uint (value, skey->ktype);
         break;
@@ -162,6 +172,14 @@ seahorse_key_class_init (SeahorseKeyClass *klass)
         g_param_spec_uint ("key-id", "Key ID", "Key identifier", 
                            0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
     
+    g_object_class_install_property (gobject_class, PROP_RAW_ID,
+        g_param_spec_string ("raw-id", "Raw ID", "Backend specific key identifier",
+                             NULL, G_PARAM_READABLE));
+                             
+    g_object_class_install_property (gobject_class, PROP_KEY_ID,
+        g_param_spec_string ("key-desc", "Key Desc", "Key Description",
+                             NULL, G_PARAM_READABLE));
+    
     g_object_class_install_property (gobject_class, PROP_KTYPE,
         g_param_spec_uint ("ktype", "Key Type", "Key type", 
                            0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
@@ -187,6 +205,7 @@ seahorse_key_class_init (SeahorseKeyClass *klass)
         NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
         
     /* Some special fields */
+    special_properties[PROP_SPECIAL_KEY_ID] = g_quark_from_static_string ("key-id");
     special_properties[PROP_SPECIAL_DISPLAY_NAME] = g_quark_from_static_string ("display-name");
     special_properties[PROP_SPECIAL_CN] = g_quark_from_static_string ("cn");
 }
@@ -261,6 +280,12 @@ seahorse_key_get_source  (SeahorseKey *skey)
     return skey->sksrc;
 }
 
+const gchar*
+seahorse_key_get_desc (SeahorseKey *skey)
+{
+    return skey->keydesc;
+}
+
 GQuark
 seahorse_key_get_ktype (SeahorseKey *skey)
 {
@@ -339,6 +364,38 @@ seahorse_key_get_name_cn (SeahorseKey *skey, guint uid)
     return (*klass->get_name_cn) (skey, uid);
 }
 
+gchar*
+seahorse_key_get_name_markup (SeahorseKey *skey, guint uid)
+{
+    SeahorseKeyClass *klass;
+    gchar *name, *markup;
+    
+    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), NULL);
+    klass = SEAHORSE_KEY_GET_CLASS (skey);
+    
+    /* If not implemented, then just return the normal name */
+    if (!klass->get_name_markup) {
+        name = seahorse_key_get_name (skey, uid);
+        markup = g_markup_escape_text (name, -1);
+        g_free (name);
+        return markup;
+    }
+    
+    return (*klass->get_name_markup) (skey, uid);
+}
+
+SeahorseValidity  
+seahorse_key_get_name_validity  (SeahorseKey *skey, guint uid)
+{
+    SeahorseKeyClass *klass;
+    
+    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), SEAHORSE_VALIDITY_UNKNOWN);
+    klass = SEAHORSE_KEY_GET_CLASS (skey);
+    g_return_val_if_fail (klass->get_name_validity, SEAHORSE_VALIDITY_UNKNOWN);
+    
+    return (*klass->get_name_validity) (skey, uid);
+}
+
 gchar*
 seahorse_key_get_display_name (SeahorseKey *skey)
 {
@@ -347,6 +404,14 @@ seahorse_key_get_display_name (SeahorseKey *skey)
     return name;    
 }
 
+gchar*
+seahorse_key_get_display_id (SeahorseKey *skey)
+{
+    gchar *id;
+    g_object_get (skey, "display-id", &id, NULL);
+    return id;
+}
+
 gchar*
 seahorse_key_get_simple_name (SeahorseKey *skey)
 {
@@ -395,6 +460,14 @@ seahorse_key_get_length (SeahorseKey *skey)
     return length;
 }
 
+const gchar*
+seahorse_key_get_stock_id (SeahorseKey *skey)
+{
+    const gchar* stock_id;
+    g_object_get (skey, "stock-id", &stock_id, NULL);
+    return stock_id;
+}
+
 void
 seahorse_key_set_preferred (SeahorseKey *skey, SeahorseKey *preferred)
 {
@@ -415,22 +488,31 @@ seahorse_key_lookup_property (SeahorseKey *skey, guint uid, const gchar *field,
 {
     GParamSpec *spec;
     GQuark qfield;
-    gchar *name;
+    gchar *text;
+
+    qfield = g_quark_from_string (field);
+    
+    /* The key-id for these guys contains the uid */
+    if (qfield == special_properties[PROP_SPECIAL_KEY_ID]) {
+        text = seahorse_context_key_to_dbus (SCTX_APP (), skey, uid);
+        g_value_init (value, G_TYPE_STRING);
+        g_value_take_string (value, text);
+        return TRUE;
+    }
     
     /* Special UID fields */
     if (uid > 0) {
 
-        qfield = g_quark_from_string (field);
         if (qfield == special_properties[PROP_SPECIAL_DISPLAY_NAME]) {
-            name = seahorse_key_get_name (skey, uid);
+            text = seahorse_key_get_name (skey, uid);
             g_value_init (value, G_TYPE_STRING);
-            g_value_take_string (value, name ? name : g_strdup (""));
+            g_value_take_string (value, text ? text : g_strdup (""));
             return TRUE;
 
         } else if (qfield == special_properties[PROP_SPECIAL_CN]) {
-            name = seahorse_key_get_name_cn (skey, uid);
+            text = seahorse_key_get_name_cn (skey, uid);
             g_value_init (value, G_TYPE_STRING);
-            g_value_take_string (value, name ? name : g_strdup (""));
+            g_value_take_string (value, text ? text : g_strdup (""));
             return TRUE;
         }
     }
@@ -441,6 +523,7 @@ seahorse_key_lookup_property (SeahorseKey *skey, guint uid, const gchar *field,
 
     g_value_init (value, spec->value_type);
     g_object_get_property (G_OBJECT (skey), field, value);
+    
     return TRUE; 
 }
 
