@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004 - 2006 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,24 @@
  * 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
+#include "config.h"
 #include <stdlib.h>
 #include <libintl.h>
 #include <gnome.h>
 #include <ldap.h>
 
+#ifdef WITH_SOUP
+#include <libsoup/soup-address.h>
+#endif
+
 #include "config.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-ldap-source.h"
 #include "seahorse-util.h"
 #include "seahorse-pgp-key.h"
+#include "seahorse-vfs-data.h"
 
 #ifdef WITH_LDAP
 
@@ -109,14 +115,36 @@ get_ldap_server_info (SeahorseLDAPSource *lsrc, gboolean force)
  
 #define LDAP_ERROR_DOMAIN (get_ldap_error_domain())
 
+static gchar**
+get_ldap_values (LDAP *ld, LDAPMessage *entry, const char *attribute)
+{
+    GArray *array;
+    struct berval **bv;
+    gchar *value;
+    int num, i;
+
+    bv = ldap_get_values_len (ld, entry, attribute);
+    if (!bv)
+        return NULL;
+
+    array = g_array_new (TRUE, TRUE, sizeof (gchar*));
+    num = ldap_count_values_len (bv);
+    for(i = 0; i < num; i++) {
+        value = g_strndup (bv[i]->bv_val, bv[i]->bv_len);
+        g_array_append_val(array, value);
+    }
+
+    return (gchar**)g_array_free (array, FALSE);
+}
+
 #if DEBUG_LDAP_ENABLE
 
 static void
 dump_ldap_entry (LDAP *ld, LDAPMessage *res)
 {
     BerElement *pos;
-    char **values;
-    char **v;
+    gchar **values;
+    gchar **v;
     char *t;
     
     t = ldap_get_dn (ld, res);
@@ -126,11 +154,11 @@ dump_ldap_entry (LDAP *ld, LDAPMessage *res)
     for (t = ldap_first_attribute (ld, res, &pos); t; 
          t = ldap_next_attribute (ld, res, pos)) {
              
-        values = ldap_get_values (ld, res, t);
+        values = get_ldap_values (ld, res, t);
         for (v = values; *v; v++) 
             g_printerr ("%s: %s\n", t, *v);
              
-        ldap_value_free (values);
+        g_strfreev (values);
         ldap_memfree (t);
     }
     
@@ -149,44 +177,44 @@ get_ldap_error_domain ()
 }
 
 static gchar*
-get_string_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
+get_string_attribute (LDAP *ld, LDAPMessage *res, const char *attribute)
 {
-    char **vals;
+    gchar **vals;
     gchar *v;
     
-    vals = ldap_get_values (ld, res, attribute);
+    vals = get_ldap_values (ld, res, attribute);
     if (!vals)
         return NULL; 
     v = vals[0] ? g_strdup (vals[0]) : NULL;
-    ldap_value_free (vals);
+    g_strfreev (vals);
     return v;
 }
 
 static gboolean
 get_boolean_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
 {
-    char **vals;
+    gchar **vals;
     gboolean b;
     
-    vals = ldap_get_values (ld, res, attribute);
+    vals = get_ldap_values (ld, res, attribute);
     if (!vals)
         return FALSE;
     b = vals[0] && atoi (vals[0]) == 1;
-    ldap_value_free (vals);
+    g_strfreev (vals);
     return b;
 }
 
 static long int
 get_int_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
 {
-    char **vals;
+    gchar **vals;
     long int d;
     
-    vals = ldap_get_values (ld, res, attribute);
+    vals = get_ldap_values (ld, res, attribute);
     if (!vals)
         return 0;
     d = vals[0] ? atoi (vals[0]) : 0;
-    ldap_value_free (vals);
+    g_strfreev (vals);
     return d;         
 }
 
@@ -194,10 +222,10 @@ static long int
 get_date_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
 {
     struct tm t;
-    char **vals;
+    gchar **vals;
     long int d;
     
-    vals = ldap_get_values (ld, res, attribute);
+    vals = get_ldap_values (ld, res, attribute);
     if (!vals)
         return 0;
         
@@ -216,7 +244,7 @@ get_date_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
         d = mktime (&t);
     }        
 
-    ldap_value_free (vals);
+    g_strfreev (vals);
     return d;         
 }
 
@@ -224,9 +252,9 @@ static gpgme_pubkey_algo_t
 get_algo_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
 {
     gpgme_pubkey_algo_t a = 0;
-    char **vals;
+    gchar **vals;
     
-    vals = ldap_get_values (ld, res, attribute);
+    vals = get_ldap_values (ld, res, attribute);
     if (!vals)
         return 0;
     
@@ -241,7 +269,7 @@ get_algo_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
             a = GPGME_PK_DSA;     
     }
     
-    ldap_value_free (vals);
+    g_strfreev (vals);
     return a;
 }
 
@@ -288,6 +316,9 @@ escape_ldap_value (const gchar *v)
 typedef gboolean (*OpLDAPCallback)   (SeahorseOperation *op, LDAPMessage *result);
     
 DECLARE_OPERATION (LDAP, ldap)
+#ifdef WITH_SOUP
+    SoupAddress *addr;              /* For async DNS lookup */
+#endif
     SeahorseLDAPSource *lsrc;       /* The source */
     LDAP *ldap;                     /* The LDAP connection */
     int ldap_op;                    /* The current LDAP async msg */
@@ -320,14 +351,26 @@ static void
 seahorse_ldap_operation_dispose (GObject *gobject)
 {
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (gobject);
-    
+
+#ifdef WITH_SOUP
+    if (lop->addr)
+        g_object_unref (lop->addr);
+    lop->addr = NULL;
+#endif 
+        
     if (lop->lsrc) {
         g_object_unref (lop->lsrc);
         lop->lsrc = NULL;
     }
+    
+    if (lop->ldap_op != -1) {
+        if (lop->ldap)
+            ldap_abandon_ext (lop->ldap, lop->ldap_op, NULL, NULL);
+        lop->ldap_op = -1;
+    }
 
     if (lop->ldap) {
-        ldap_unbind (lop->ldap);
+        ldap_unbind_ext (lop->ldap, NULL, NULL);
         lop->ldap = NULL;
     }
     
@@ -360,14 +403,21 @@ seahorse_ldap_operation_cancel (SeahorseOperation *operation)
     g_assert (SEAHORSE_IS_LDAP_OPERATION (operation));
     lop = SEAHORSE_LDAP_OPERATION (operation);
     
+#ifdef WITH_SOUP
+    /* This cancels lookup */
+    if (lop->addr)
+        g_object_unref (lop->addr);
+    lop->addr = NULL;
+#endif
+    
     if (lop->ldap_op != -1) {
         if (lop->ldap)
-            ldap_abandon (lop->ldap, lop->ldap_op);
+            ldap_abandon_ext (lop->ldap, lop->ldap_op, NULL, NULL);
         lop->ldap_op = -1;
     }
 
     if (lop->ldap) {
-        ldap_unbind (lop->ldap);
+        ldap_unbind_ext (lop->ldap, NULL, NULL);
         lop->ldap = NULL;
     }
 
@@ -404,21 +454,23 @@ result_callback (SeahorseLDAPOperation *lop)
     gboolean ret;
     int r, i;
     
-    g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
-    g_assert (lop->ldap != NULL);
-    g_assert (lop->ldap_op != -1);
-    
     for (i = 0; i < DEFAULT_LOAD_BATCH; i++) {
-     
+
+        g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
+        g_assert (lop->ldap != NULL);
+        g_assert (lop->ldap_op != -1);
+        
         /* This effects a poll */
         timeout.tv_sec = 0;
         timeout.tv_usec = 0;
-    
+
         r = ldap_result (lop->ldap, lop->ldap_op, 0, &timeout, &result);
         switch (r) {   
-        case -1: /* Scary error */
-            g_return_val_if_reached (FALSE);
-            break;
+        case -1: /* Strange system error */
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
+                g_error_new (LDAP_ERROR_DOMAIN, errno, strerror(errno)));
+            return FALSE;
+        
         case 0: /* Timeout exceeded */
             return TRUE;
         };
@@ -437,7 +489,7 @@ result_callback (SeahorseLDAPOperation *lop)
         g_assert (lop->ldap_cb);
 
         /* Should not be marked as done. */
-        g_assert (!seahorse_operation_is_done (SEAHORSE_OPERATION (lop)));
+        g_assert (seahorse_operation_is_running (SEAHORSE_OPERATION (lop)));
     }    
         
     return ret;
@@ -475,7 +527,7 @@ done_info_start_op (SeahorseOperation *op, LDAPMessage *result)
             sinfo->key_attr = g_strdup (sinfo->version > 1 ? "pgpkeyv2" : "pgpkey");
             set_ldap_server_info (lop->lsrc, sinfo);
             
-            ldap_abandon (lop->ldap, lop->ldap_op);
+            ldap_abandon_ext (lop->ldap, lop->ldap_op, NULL, NULL);
             lop->ldap_op = -1;
             
         } else {
@@ -530,10 +582,11 @@ done_bind_start_info (SeahorseOperation *op, LDAPMessage *result)
         return done_info_start_op (op, NULL);
         
     /* Retrieve the server info */
-    lop->ldap_op = ldap_search (lop->ldap, "cn=PGPServerInfo", LDAP_SCOPE_BASE,
-                                "(objectclass=*)", (char**)kServerAttributes, 0);    
-    if (lop->ldap_op == -1) {
-        fail_ldap_operation (lop, 0);
+    r = ldap_search_ext (lop->ldap, "cn=PGPServerInfo", LDAP_SCOPE_BASE,
+                         "(objectclass=*)", (char**)kServerAttributes, 0,
+                         NULL, NULL, NULL, 0, &(lop->ldap_op));    
+    if (r != LDAP_SUCCESS) {
+        fail_ldap_operation (lop, r);
         return FALSE;
     }
 
@@ -541,34 +594,61 @@ done_bind_start_info (SeahorseOperation *op, LDAPMessage *result)
     return TRUE;
 }
 
-/* Start an LDAP (bind, server info) request */
-static SeahorseLDAPOperation*
-seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
-                               guint total)
+/* Once the DNS name is resolved, we end up here */
+static void
+resolved_callback (gpointer unused, guint status, SeahorseLDAPOperation *lop)
 {
-    SeahorseLDAPOperation *lop;
+    guint port = LDAP_PORT;
     gchar *server = NULL;
     gchar *t;
-
-    g_assert (SEAHORSE_IS_LDAP_SOURCE (lsrc));
     
-    lop = g_object_new (SEAHORSE_TYPE_LDAP_OPERATION, NULL);
-    lop->lsrc = lsrc;
-    g_object_ref (lsrc);
+    g_object_get (lop->lsrc, "key-server", &server, NULL);
+    g_return_if_fail (server && server[0]);
     
-    g_object_get (lsrc, "key-server", &server, NULL);
-    g_return_val_if_fail (server && server[0], NULL);
+    if ((t = strchr (server, ':')) != NULL) {
+        *t = 0;
+        t++;
+        port = atoi (t);
+        if (port <= 0 || port >= G_MAXUINT16) {
+            g_warning ("invalid port number: %s (using default)", t);
+            port = LDAP_PORT;
+        }
+    }
+
+#ifdef WITH_SOUP
     
-    lop->ldap = ldap_init (server, LDAP_PORT);
-    g_return_val_if_fail (lop->ldap != NULL, NULL);
-
-    lop->ldap_cb = done_bind_start_info;
-    lop->chain_cb = cb;
-
-    seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
+    /* DNS failed */
+    if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
+        seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
+            g_error_new (SEAHORSE_ERROR, -1, _("Couldn't resolve address: %s"), server));
+        g_free (server);
+        return;
+    } 
+    
+    g_assert (lop->addr);
+    
+    {
+        /* Now that we've resolved our address, connect via IP */
+        const char *ip;
+        ip = soup_address_get_physical (lop->addr);
+        g_return_if_fail (ip != NULL);
+        
+        lop->ldap = ldap_init (ip, port);
+        g_return_if_fail (lop->ldap != NULL);
+    }
+    
+#else /* WITH_SOUP */
+    
+    /* No async DNS resolve, let libldap handle resolving synchronously */
+    lop->ldap = ldap_init (server, port);
+    g_return_if_fail (lop->ldap != NULL);    
+    
+#endif /* WITH_SOUP */
+    
+    /* The ldap_cb and chain_cb were set in seahorse_ldap_operation_start */
     
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
     g_free (t);
 
     g_free (server);
@@ -581,7 +661,55 @@ seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
     else   /* This starts looking for results */
         lop->stag = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, 
                         (GSourceFunc)result_callback, lop, NULL);
-        
+}
+
+/* Start an LDAP (bind, server info) request */
+static SeahorseLDAPOperation*
+seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
+                               guint total)
+{
+    SeahorseLDAPOperation *lop;
+#ifdef WITH_SOUP
+    gchar *server = NULL;
+    gchar *t;
+#endif
+
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (lsrc));
+    
+    lop = g_object_new (SEAHORSE_TYPE_LDAP_OPERATION, NULL);
+    lop->lsrc = lsrc;
+    g_object_ref (lsrc);
+
+    /* Not used until step after resolve */
+    lop->ldap_cb = done_bind_start_info;
+    lop->chain_cb = cb;
+    
+    seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
+    
+#ifdef WITH_SOUP
+    
+    /* Try and resolve asynchronously */
+    g_object_get (lsrc, "key-server", &server, NULL);
+    g_return_val_if_fail (server && server[0], NULL);
+
+    if ((t = strchr (server, ':')) != NULL)
+        *t = 0;
+    lop->addr = soup_address_new (server, LDAP_PORT);
+    
+    t = g_strdup_printf (_("Resolving server address: %s"), server);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
+    g_free (t);
+
+    g_free (server);
+    
+    soup_address_resolve_async (lop->addr, (SoupAddressCallback)resolved_callback, lop);
+    
+#else /* no WITH_SOUP */
+    
+    resolved_callback (NULL, 0, lop);
+    
+#endif
+    
     return lop;
 }
 
@@ -699,6 +827,7 @@ start_search (SeahorseOperation *op, LDAPMessage *result)
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (op);  
     LDAPServerInfo *sinfo;
     gchar *filter, *t;
+    int r;
     
     g_return_val_if_fail (lop->ldap != NULL, FALSE);
     g_assert (lop->ldap_op == -1);
@@ -707,13 +836,14 @@ start_search (SeahorseOperation *op, LDAPMessage *result)
     g_return_val_if_fail (filter != NULL, FALSE);
 
     t = (gchar*)g_object_get_data (G_OBJECT (lop), "details");
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, 0);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
     
     sinfo = get_ldap_server_info (lop->lsrc, TRUE);
-    lop->ldap_op = ldap_search (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
-                                filter, (char**)kPGPAttributes, 0);
-    if (lop->ldap_op == -1) {
-        fail_ldap_operation (lop, 0);
+    r = ldap_search_ext (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
+                         filter, (char**)kPGPAttributes, 0,
+                         NULL, NULL, NULL, 0, &(lop->ldap_op));    
+    if (r != LDAP_SUCCESS) {
+        fail_ldap_operation (lop, r);
         return FALSE;
     }                                    
     
@@ -787,6 +917,8 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
     LDAPServerInfo *sinfo;
     gpgme_data_t data;
     char *message;
+    GError *err = NULL;
+    gboolean ret;
     gchar *key;
     int code;
     int r;
@@ -809,16 +941,19 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
             fail_ldap_operation (lop, LDAP_NO_SUCH_OBJECT); 
         }
         
-        data = (gpgme_data_t)g_object_get_data (G_OBJECT (lop), "result");
+        data = (gpgme_data_t)seahorse_operation_get_result (SEAHORSE_OPERATION (lop));
         g_return_val_if_fail (data != NULL, FALSE);
         
-        r = gpgme_data_write (data, key, strlen (key));
-        g_return_val_if_fail (r != -1, FALSE);
-
-        r = gpgme_data_write (data, "\n", 1);
-        g_return_val_if_fail (r != -1, FALSE);
-
+        ret = seahorse_vfs_data_write_all (data, key, -1, &err) &&
+              seahorse_vfs_data_write_all (data, "\n", -1, &err);
+        
         g_free (key);
+        
+        if (!ret) {
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, err);
+            return FALSE;
+        }
+        
         return TRUE;
 
     /* No more entries, result */
@@ -853,7 +988,7 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     gchar *filter;
     char *attrs[2];
     const gchar *fpr;
-    int l;
+    int l, r;
     
     g_assert (lop->ldap != NULL);
     g_assert (lop->ldap_op == -1);
@@ -862,9 +997,9 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     fprfull = (GSList*)g_object_get_data (G_OBJECT (lop), "fingerprints-full");
 
     l = g_slist_length (fprfull);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
-                                      _("Retrieving remote keys..."), 
-                                      l - g_slist_length (fingerprints), l);
+    seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (lop), 
+                                           _("Retrieving remote keys..."), 
+                                           l - g_slist_length (fingerprints), l);
     
     if (fingerprints) {
      
@@ -885,13 +1020,13 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
         attrs[0] = sinfo->key_attr;
         attrs[1] = NULL;
         
-        lop->ldap_op = ldap_search (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
-                                    filter, attrs, 0);
-
+        r = ldap_search_ext (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
+                             filter, attrs, 0,
+                             NULL, NULL, NULL, 0, &(lop->ldap_op));
         g_free (filter);
 
-        if (lop->ldap_op == -1) {
-            fail_ldap_operation (lop, 0);
+        if (r != LDAP_SUCCESS) {
+            fail_ldap_operation (lop, r);
             return FALSE;
         }                                    
                 
@@ -910,7 +1045,6 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
                               gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
-    gpgme_error_t gerr;
     
     g_assert (g_slist_length (fingerprints) > 0);
     
@@ -920,13 +1054,12 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
         
     if (data) {
         /* Note that we don't auto-free this */
-        g_object_set_data (G_OBJECT (lop), "result", data);
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (lop), data, NULL);
     } else {
         /* But when we auto create a data object then we free it */
-        gerr = gpgme_data_new (&data);
-        g_return_val_if_fail (data != NULL, NULL);
-        g_object_set_data_full (G_OBJECT (lop), "result", data, 
-                                (GDestroyNotify)gpgme_data_release);
+        data = gpgmex_data_new ();
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (lop), data, 
+                                        (GDestroyNotify)gpgmex_data_release);
     }
     
     g_object_set_data (G_OBJECT (lop), "fingerprints", fingerprints);
@@ -988,6 +1121,7 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
     LDAPMod *attrs[2];
     char *values[2];
     guint l;
+    int r;
 
     g_assert (lop->ldap != NULL);
     g_assert (lop->ldap_op == -1);
@@ -996,9 +1130,9 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
     keysfull = (GSList*)g_object_get_data (G_OBJECT (lop), "key-data-full");
     
     l = g_slist_length (keysfull);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
-                                      _("Sending keys to key server..."), 
-                                      l - g_slist_length (keys), l);
+    seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (lop), 
+                                           _("Sending keys to key server..."), 
+                                           l - g_slist_length (keys), l);
     
     if (keys) {
      
@@ -1024,12 +1158,12 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
         
         base = g_strdup_printf ("pgpCertid=virtual,%s", sinfo->base_dn);
         
-        lop->ldap_op = ldap_add (lop->ldap, base, attrs);
+        r = ldap_add_ext (lop->ldap, base, attrs, NULL, NULL, &(lop->ldap_op));
 
         g_free (base);
-                
-        if (lop->ldap_op == -1) {
-            fail_ldap_operation (lop, 0);
+        
+        if (r != LDAP_SUCCESS) {
+            fail_ldap_operation (lop, r);
             return FALSE;
         }                                    
                 
@@ -1065,59 +1199,17 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
  *  SEAHORSE LDAP SOURCE
  */
  
-/* GObject handlers */
-static void seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass);
+G_DEFINE_TYPE (SeahorseLDAPSource, seahorse_ldap_source, SEAHORSE_TYPE_SERVER_SOURCE);
 
-/* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_ldap_source_load      (SeahorseKeySource *src,
-                                                           SeahorseKeySourceLoad load,
-                                                           const gchar *match);
-static SeahorseOperation*  seahorse_ldap_source_import    (SeahorseKeySource *sksrc, 
-                                                           gpgme_data_t data);
-static SeahorseOperation*  seahorse_ldap_source_export    (SeahorseKeySource *sksrc, 
-                                                           GList *keys,     
-                                                           gboolean complete,
-                                                           gpgme_data_t data);
-                                                           
-static SeahorseKeySourceClass *parent_class = NULL;
 
-GType
-seahorse_ldap_source_get_type (void)
+static void 
+seahorse_ldap_source_init (SeahorseLDAPSource *lsrc)
 {
-    static GType type = 0;
- 
-    if (!type) {
-        
-        static const GTypeInfo tinfo = {
-            sizeof (SeahorseLDAPSourceClass), NULL, NULL,
-            (GClassInitFunc) seahorse_ldap_source_class_init, NULL, NULL,
-            sizeof (SeahorseLDAPSource), 0, NULL
-        };
-        
-        type = g_type_register_static (SEAHORSE_TYPE_SERVER_SOURCE, 
-                                       "SeahorseLDAPSource", &tinfo, 0);
-    }
-  
-    return type;
-}
 
-/* Initialize the basic class stuff */
-static void
-seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
-{
-    SeahorseKeySourceClass *key_class;
-   
-    key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
-    key_class->load = seahorse_ldap_source_load;
-    key_class->import = seahorse_ldap_source_import;
-    key_class->export = seahorse_ldap_source_export;
-
-    parent_class = g_type_class_peek_parent (klass);
 }
 
 static SeahorseOperation*
-seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
-                           const gchar *match)
+seahorse_ldap_source_load (SeahorseKeySource *src, GQuark keyid)
 {
     SeahorseOperation *op;
     SeahorseLDAPOperation *lop = NULL;
@@ -1125,22 +1217,35 @@ seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     g_assert (SEAHORSE_IS_KEY_SOURCE (src));
     g_assert (SEAHORSE_IS_LDAP_SOURCE (src));
 
-    op = parent_class->load (src, load, match);
+    op = SEAHORSE_KEY_SOURCE_CLASS (seahorse_ldap_source_parent_class)->load (src, keyid);
     if (op != NULL)
         return op;
     
     /* No way to find new or all keys */
-    if (load == SKSRC_LOAD_NEW || load == SKSRC_LOAD_ALL) 
+    if (!keyid) 
         return seahorse_operation_new_complete (NULL);
 
+    lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), 
+                                      seahorse_key_get_rawid (keyid));
+    g_return_val_if_fail (lop != NULL, NULL);
+
+    seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src),
+                                           SEAHORSE_OPERATION (lop));
+    g_object_ref (lop);
+    return SEAHORSE_OPERATION (lop);
+}
+
+static SeahorseOperation*
+seahorse_ldap_source_search (SeahorseKeySource *src, const gchar *match)
+{
+    SeahorseLDAPOperation *lop = NULL;
+
+    g_assert (SEAHORSE_IS_KEY_SOURCE (src));
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (src));
+
     /* Search for keys */
-    else if (load == SKSRC_LOAD_SEARCH)
-        lop = start_search_operation (SEAHORSE_LDAP_SOURCE (src), match);
-        
-    /* Load a specific key */
-    else if (load == SKSRC_LOAD_KEY)
-        lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), match);
-    
+    lop = start_search_operation (SEAHORSE_LDAP_SOURCE (src), match);
+     
     g_return_val_if_fail (lop != NULL, NULL);
     seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src),
                                            SEAHORSE_OPERATION (lop));
@@ -1179,26 +1284,24 @@ seahorse_ldap_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     lop = start_send_operation_multiple (lsrc, keydata);
     g_return_val_if_fail (lop != NULL, NULL);
     
+    gpgmex_data_release (data);
     return SEAHORSE_OPERATION (lop);
 }
 
 static SeahorseOperation* 
-seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                             gboolean complete, gpgme_data_t data)
+seahorse_ldap_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids, 
+                                 gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
     SeahorseLDAPSource *lsrc;
-    GSList *fingerprints = NULL;
+    GSList *l, *fingerprints = NULL;
     
     g_assert (SEAHORSE_IS_LDAP_SOURCE (sksrc));
     lsrc = SEAHORSE_LDAP_SOURCE (sksrc);
     
-    for ( ; keys; keys = g_list_next (keys)) {
-        g_assert (SEAHORSE_IS_KEY (keys->data));
-        fingerprints = g_slist_prepend (fingerprints,
-                g_strdup (seahorse_key_get_keyid (SEAHORSE_KEY (keys->data))));
-    }
-    
+    for (l = keyids; l; l = g_slist_next (l)) 
+        fingerprints = g_slist_prepend (fingerprints, 
+            g_strdup (seahorse_key_get_rawid (GPOINTER_TO_UINT (l->data))));
     fingerprints = g_slist_reverse (fingerprints);
 
     lop = start_get_operation_multiple (lsrc, fingerprints, data);
@@ -1207,6 +1310,22 @@ seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys,
     return SEAHORSE_OPERATION (lop);    
 }
 
+/* Initialize the basic class stuff */
+static void
+seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
+{
+    SeahorseKeySourceClass *key_class;
+   
+    key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
+    key_class->load = seahorse_ldap_source_load;
+    key_class->search = seahorse_ldap_source_search;
+    key_class->import = seahorse_ldap_source_import;
+    key_class->export_raw = seahorse_ldap_source_export_raw;
+
+    seahorse_ldap_source_parent_class = g_type_class_peek_parent (klass);
+}
+
+
 /**
  * seahorse_ldap_source_new
  * @uri: The server to connect to 
