@@ -78,8 +78,21 @@ utf8_validate (const gchar *text)
     return result;
 }
 
+static gboolean
+key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+    /* Close the dialog when hitting "Esc". */
+    if (event->keyval == GDK_Escape)
+    {
+        gtk_dialog_response(GTK_DIALOG (widget), GTK_RESPONSE_REJECT);
+        return TRUE;
+    }
+    
+    return FALSE;
+}
+
 static void
-grab_keyboard (GtkWidget *win, GdkEvent * event, gpointer data)
+grab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 {
 #ifndef _DEBUG
     if (gdk_keyboard_grab (win->window, FALSE, gdk_event_get_time (event)))
@@ -96,6 +109,15 @@ ungrab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 #endif
 }
 
+/* When enter is pressed in the confirm entry, move */
+static void
+confirm_callback (GtkWidget *widget, GtkDialog *dialog)
+{
+    GtkWidget *entry = GTK_WIDGET (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
+    g_assert (SEAHORSE_IS_SECURE_ENTRY (entry));
+    gtk_widget_grab_focus (entry);
+}
+
 /* When enter is pressed in the entry, we simulate an ok */
 static void
 enter_callback (GtkWidget *widget, GtkDialog *dialog)
@@ -103,6 +125,18 @@ enter_callback (GtkWidget *widget, GtkDialog *dialog)
     gtk_dialog_response (dialog, GTK_RESPONSE_ACCEPT);
 }
 
+static void
+entry_changed (GtkEditable *editable, GtkDialog *dialog)
+{
+    SeahorseSecureEntry *entry, *confirm;
+    
+    entry = SEAHORSE_SECURE_ENTRY (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
+    confirm = SEAHORSE_SECURE_ENTRY (g_object_get_data (G_OBJECT (dialog), "confirm-entry"));
+    
+    gtk_dialog_set_response_sensitive (dialog, GTK_RESPONSE_ACCEPT, 
+                                       strcmp (seahorse_secure_entry_get_text (entry), 
+                                               seahorse_secure_entry_get_text (confirm)) == 0);
+}
 
 static void
 constrain_size (GtkWidget *win, GtkRequisition *req, gpointer data)
@@ -124,13 +158,15 @@ constrain_size (GtkWidget *win, GtkRequisition *req, gpointer data)
 
 GtkDialog*
 seahorse_passphrase_prompt_show (const gchar *title, const gchar *description, 
-                                 const gchar *prompt, const gchar *errmsg)
+                                 const gchar *prompt, const gchar *check,
+                                 gboolean confirm)
 {
     SeahorseSecureEntry *entry;
     GtkDialog *dialog;
     GtkWidget *w;
     GtkWidget *box;
     GtkWidget *ebox;
+    GtkTable *table;
     GtkWidget *wvbox;
     GtkWidget *chbox;
     gchar *msg;
@@ -176,40 +212,55 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
         gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
     }
 
-    /* Any error messsages */
-    if (errmsg) {
-        GdkColor color = { 0, 0xffff, 0, 0 };
+    /* Two entries (usually on is hidden)  in a vbox */
+    table = GTK_TABLE (gtk_table_new (3, 2, FALSE));
+    gtk_table_set_row_spacings (table, HIG_SMALL);
+    gtk_table_set_col_spacings (table, HIG_LARGE);
+    gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (table), FALSE, FALSE, 0);
 
-        msg = utf8_validate (errmsg);
+    /* The first entry if we have one */
+    if (confirm) {
+        ebox = gtk_hbox_new (FALSE, HIG_LARGE);
+        msg = utf8_validate (prompt);
         w = gtk_label_new (msg);
         g_free (msg);
+        gtk_table_attach (table, w, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
 
-        gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-        gtk_label_set_line_wrap (GTK_LABEL (w), TRUE);
-        gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
-        gtk_widget_modify_fg (w, GTK_STATE_NORMAL, &color);
+        entry = SEAHORSE_SECURE_ENTRY (seahorse_secure_entry_new ());
+        gtk_widget_set_size_request (GTK_WIDGET (entry), 200, -1);
+        g_object_set_data (G_OBJECT (dialog), "confirm-entry", entry);
+        g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (confirm_callback), dialog);
+        g_signal_connect (G_OBJECT (entry), "changed", G_CALLBACK (entry_changed), dialog);
+        gtk_table_attach_defaults (table, GTK_WIDGET (entry), 1, 2, 0, 1);
+        gtk_widget_grab_focus (GTK_WIDGET (entry));
     }
 
-    ebox = gtk_hbox_new (FALSE, HIG_SMALL);
-    gtk_box_pack_start (GTK_BOX (box), ebox, FALSE, FALSE, 0);
-
-    /* Prompt goes before the entry */
-    msg = utf8_validate (prompt);
+    /* The second and main entry */
+    ebox = gtk_hbox_new (FALSE, HIG_LARGE);
+    msg = utf8_validate (confirm ? _("Confirm:") : prompt);
     w = gtk_label_new (msg);
     g_free (msg);
-
-    gtk_box_pack_start (GTK_BOX (ebox), w, FALSE, FALSE, 0);
-
+    gtk_table_attach (table, w, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+    
     entry = SEAHORSE_SECURE_ENTRY (seahorse_secure_entry_new ());
     gtk_widget_set_size_request (GTK_WIDGET (entry), 200, -1);
-    g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (enter_callback), dialog);
-
-    gtk_box_pack_start (GTK_BOX (ebox), GTK_WIDGET (entry), TRUE, TRUE, 0);
-    gtk_widget_grab_focus (GTK_WIDGET (entry));
-    gtk_widget_show (GTK_WIDGET (entry));
-
     g_object_set_data (G_OBJECT (dialog), "secure-entry", entry);
+    g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (enter_callback), dialog);
+    gtk_table_attach_defaults (table, GTK_WIDGET (entry), 1, 2, 1, 2);
+    if (!confirm)
+        gtk_widget_grab_focus (GTK_WIDGET (entry));
+    else
+        g_signal_connect (G_OBJECT (entry), "changed", G_CALLBACK (entry_changed), dialog);
 
+    /* The checkbox */
+    if (check) {
+        w = gtk_check_button_new_with_mnemonic (check);
+        gtk_table_attach_defaults (table, w, 1, 2, 2, 3);
+        g_object_set_data (G_OBJECT (dialog), "check-option", w);
+    }
+
+    gtk_widget_show_all (GTK_WIDGET (table));
+    
     w = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
     gtk_dialog_add_action_widget (dialog, w, GTK_RESPONSE_REJECT);
     GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
@@ -220,12 +271,17 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
     g_signal_connect_object (G_OBJECT (entry), "focus_in_event",
                              G_CALLBACK (gtk_widget_grab_default), G_OBJECT (w), 0);
     gtk_widget_grab_default (w);
+    
+    g_signal_connect (dialog, "key_press_event", G_CALLBACK (key_press), NULL);
 
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
     gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
     gtk_window_present (GTK_WINDOW (dialog));
     gtk_widget_show_all (GTK_WIDGET (dialog));
 
+    if (confirm)
+        entry_changed (NULL, dialog);
+    
     return dialog;
 }
 
@@ -238,6 +294,14 @@ seahorse_passphrase_prompt_get (GtkDialog *dialog)
     return seahorse_secure_entry_get_text (entry);
 }
 
+gboolean
+seahorse_passphrase_prompt_checked (GtkDialog *dialog)
+{
+    GtkToggleButton *button = GTK_TOGGLE_BUTTON (g_object_get_data (G_OBJECT (dialog), "check-option"));
+    return GTK_IS_TOGGLE_BUTTON (button) ? gtk_toggle_button_get_active (button) : FALSE;
+}
+
+
 gpgme_error_t
 seahorse_passphrase_get (gconstpointer dummy, const gchar *passphrase_hint, 
                          const char* passphrase_info, int flags, int fd)
@@ -272,7 +336,8 @@ seahorse_passphrase_get (gconstpointer dummy, const gchar *passphrase_hint,
 
     g_strfreev (split_uid);
 
-    dialog = seahorse_passphrase_prompt_show (NULL, label, NULL, errmsg);
+    dialog = seahorse_passphrase_prompt_show (NULL, errmsg ? errmsg : label, 
+                                              NULL, NULL, FALSE);
     g_free (label);
     g_free (errmsg);
     
@@ -449,7 +514,7 @@ gpg_detect_agent ()
  * SSH AGENT 
  */
 
-const gchar REQ_MESSAGE[] = { 0x01, 0x00, 0x00, 0x00, SEAHORSE_SSH_PING_MSG };
+const guchar REQ_MESSAGE[] = { 0x01, 0x00, 0x00, 0x00, SEAHORSE_SSH_PING_MSG };
 const guint RESP_INDEX = 4;
 const guint RESP_VALUE = SEAHORSE_SSH_PING_MSG;
 
@@ -459,7 +524,7 @@ ssh_detect_agent ()
     SeahorseAgentType ret;
     const gchar *socketpath;
     struct sockaddr_un sunaddr;
-    gchar buf[16];
+    guchar buf[16];
     GIOChannel *io;
     int agentfd;
     gsize bytes_read = 0;
@@ -495,9 +560,9 @@ ssh_detect_agent ()
     g_io_channel_set_buffered (io, FALSE);
 
     /* Send and receive our message */
-    if (g_io_channel_write_chars (io, REQ_MESSAGE, sizeof (REQ_MESSAGE), NULL, 
+    if (g_io_channel_write_chars (io, (const gchar*)REQ_MESSAGE, sizeof (REQ_MESSAGE), NULL, 
                                   &err) == G_IO_STATUS_NORMAL) {
-        g_io_channel_read_chars (io, buf, sizeof (buf), &bytes_read, &err);
+        g_io_channel_read_chars (io, (gchar*)buf, sizeof (buf), &bytes_read, &err);
     }
     
     if (!err) {
