@@ -26,21 +26,24 @@
 #ifndef __SEAHORSE_PROGRESS_H__
 #define __SEAHORSE_PROGRESS_H__
 
-#include <gtk/gtk.h>
-
 #include "seahorse-operation.h"
+#include "seahorse-widget.h"
 #include "seahorse-context.h"
 
 /* -----------------------------------------------------------------------------
  * SEAHORSE APPBAR HOOKS
  */
 
-void          seahorse_progress_appbar_set_operation  (GtkWidget* appbar,
-                                                       SeahorseOperation *operation);
+/* Updates two glade widgets 'status' and 'progress' in the SeahorseWidget */
+void                seahorse_progress_status_set_operation      (SeahorseWidget *swidget,
+                                                                 SeahorseOperation *operation);
+
+/* Gets the operation set with above function */
+SeahorseOperation*  seahorse_progress_status_get_operation      (SeahorseWidget *swidget);
 
 /* Keeps operation refed until progress dialog goes away */
-void          seahorse_progress_show                  (SeahorseOperation *operation,
-                                                       const gchar *title,
-                                                       gboolean delayed);
+void                seahorse_progress_show                      (SeahorseOperation *operation,
+                                                                 const gchar *title,
+                                                                 gboolean delayed);
 
 #endif /* __SEAHORSE_PROGRESS_H__ */
