@@ -132,7 +132,7 @@ load_gkeyring_items (SeahorseWidget *swidget, SeahorseContext *sctx)
     
     gsrc = seahorse_gkeyring_source_new (NULL);
     seahorse_context_take_key_source (sctx, SEAHORSE_KEY_SOURCE (gsrc));
-    op = seahorse_key_source_load (SEAHORSE_KEY_SOURCE (gsrc), SKSRC_LOAD_ALL, 0, NULL);
+    op = seahorse_key_source_load (SEAHORSE_KEY_SOURCE (gsrc), 0);
     
     /* Monitor loading progress */
     seahorse_progress_status_set_operation (swidget, op);
@@ -681,7 +681,7 @@ backup_activate (GtkWidget *widget, SeahorseWidget *swidget)
     gchar *ext, *t;
     const gchar* home_dir = NULL;
     
-    dialog = seahorse_util_chooser_save_new (_("Backup Keyrings to Archive"), 
+    dialog = seahorse_util_chooser_save_new (_("Back up Keyrings to Archive"), 
                 GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
     seahorse_util_chooser_show_archive_files (dialog);	    	    	    
 
@@ -1177,8 +1177,8 @@ static const GtkActionEntry ui_entries[] = {
             N_("Create a new personal key"), G_CALLBACK (generate_activate) },
     { "key-import-file", GTK_STOCK_OPEN, N_("_Import..."), "<control>I",
             N_("Import keys into your keyring from a file"), G_CALLBACK (import_activate) },
-    { "key-backup", GTK_STOCK_SAVE, N_("_Backup Keyrings..."), "",
-            N_("Backup all keys"), G_CALLBACK (backup_activate) }, 
+    { "key-backup", GTK_STOCK_SAVE, N_("_Back up Keyrings..."), "",
+            N_("Back up all keys"), G_CALLBACK (backup_activate) }, 
     { "key-import-clipboard", GTK_STOCK_PASTE, N_("Paste _Keys"), "<control>V",
             N_("Import keys from the clipboard"), G_CALLBACK (paste_activate) }, 
             
